/*---------------------------------------------------------------------------

    Filename: digg_import.js
    Project:  Digg Import
    (C) Copyright 2007, Kirix Corporation, All Rights Reserved.

    Summary:         Digg API Requester and Parser (Imports results into a table)
    Developer(s):    David Z. Williams
                     Jeff Engel

---------------------------------------------------------------------------*/





// global variable used to overcome eval() limitations when
// parsing JSON strings
DiggImportBase.currentResponse;


// base class used for communicating with the Digg servers
class DiggImportBase
{
    // public events
    var finished;

    // private member variables
    var m_db;
    var m_job;
    var m_job_title;
    var m_http_request;
    var m_request_type;
    var m_total_count = 0;
    var m_output_tablename = "";
    var m_append = false;
    var m_request_timer;
    var m_cancel_timer;


    function DiggImportBase(request_type)
    {
        // create a connection to the local database
        m_db = HostApp.getDatabase();

        m_request_type = request_type;
        m_job_title = "Digg Import";

        m_request_timer = new Timer;
        m_request_timer.setInterval(10000); // send a request every 10 seconds

        m_cancel_timer = new Timer;
        m_cancel_timer.setInterval(1000); // check for user cancel every second
        m_cancel_timer.tick.connect(this, onCancelTicked);

        finished = new Event;

        _initHttpRequest();
    }


    // --- "public" member functions ---

    // sets the location of where the data will be stored
    function setOutputTable(tablename)
    {
        m_output_tablename = tablename;
    }

    // enables or disables appending data to an existing table
    function setAppendData(append)
    {
        m_append = append;
    }


    // --- "protected" member functions ---

    // creates a new HttpRequest object
    function _initHttpRequest()
    {
        // create an object for web requests
        m_http_request = new HttpRequest;
        m_http_request.setUserAgent("www.kirix.com");
    }

    // parses a JSON string into an object
    function _parseResponse(response)
    {
        eval("DiggImportBase.currentResponse = "
                + response.replace(new RegExp("\"(\\w+)\":"), "$1:") + ";");

        // return the object
        return DiggImportBase.currentResponse;
    }

    // creates the base url for a request
    function _makeBaseUrl(params)
    {
        // create the base url
        var url = "http://services.digg.com/" + m_request_type
                + "?type=json&appkey=http%3A%2F%2Fwww.kirix.com";

        // add the additional parameters
        for (var k in params)
            url += "&" + k + "=" + encodeURIComponent(params[k]);

        return url;
    }


    // creates a new job to add to the job queue
    function startJob()
    {
        m_job = HostApp.createJob();
        m_job.setTitle(m_job_title);
        m_job.setMaximum(0);
        m_job.start();
        m_cancel_timer.start();
    }

    // stops timers from running and marks the job as finished
    function finishJob()
    {
        m_request_timer.stop();
        m_job.setTitle(m_job_title + " '" + m_output_tablename + "'");
        m_job.finish();
        finished.fire();
    }

    // stops timers from running and marks the job as cancelled
    function cancelJob()
    {
        m_request_timer.stop();
        m_job.setTitle(m_job_title + " '" + m_output_tablename + "'");
        m_job.cancel();
        finished.fire();
    }


    // --- "private" member functions ---

    // periodic check to see if a job was cancelled
    function onCancelTicked()
    {
        // check if the user cancelled the job
        if (m_cancel_timer.isRunning() && m_job.isCancelling())
            cancelJob();
    }

}






// class for requesting stories from digg

class DiggStoryImport extends DiggImportBase
{
    // private member variables

    var m_category;
    var m_status;
    var m_username;
    var m_overall_start_date;
    var m_overall_end_date;

    var m_lastids;
    var m_chunk_size;
    var m_request_end_date;
    var m_request_offset;
    var m_repeated;


    function DiggStoryImport()
    {
        super("stories");

        // change the settings that belong to the parent class
        m_output_tablename = "/digg/stories";
        m_chunk_size = 2592000; // default to 30 days
        m_repeated = 0;

        // set the default date range
        var d = new Date();
        m_overall_end_date = d.getTime() / 1000;
        m_overall_start_date = m_overall_end_date - m_chunk_size;

        m_request_timer.tick.connect(this, _addStoryRequest);
        m_http_request.finished.connect(this, _handleStoryResponse);
    }


    // --- "public" member functions ---

    // sets the current story container
    function setCategory(c)
    {
        c = c.replace("([A-Za-z]+)[^a-zA-Z]*([A-Za-z]*)", "$1_$2");
        if(c.charAt(c.length - 1) == "_")
            c = c.substr(0, c.length - 1);
        m_category = c.toLowerCase();
        _makeRequestType();
    }

    // sets the status filter
    function setStatus(s)
    {
        m_status = s.toLowerCase();
        _makeRequestType();
    }

    // sets the username filter
    function setUsername(u)
    {
        m_username = u;
        _makeRequestType();
    }

    // sets the minimum date
    function setStartDate(d)
    {
        var milliseconds = d.getTime();
        m_overall_start_date = (milliseconds / 1000);
    }

    // sets the maximum date
    function setEndDate(d)
    {
        var milliseconds = d.getTime();
        m_overall_end_date = (milliseconds / 1000);
    }


    // starts the import process according to the current parameters
    function run()
    {
        // create a job for this process
        startJob();

        // if not appending data, (re)create the table
        if (!m_append || !m_db.exists(m_output_tablename))
        {
            m_db.execute("DROP TABLE " + m_output_tablename + ";");
            m_db.execute("CREATE TABLE " + m_output_tablename + " ("
                    + "ID NUMERIC(8,0), SUBMIT_DATE DATETIME, "
                    + "STATUS VARCHAR(10), DIGGS NUMERIC(8,0), "
                    + "COMMENTS NUMERIC(8,0), HREF VARCHAR(150), "
                    + "LINK VARCHAR(150), TITLE VARCHAR(60), "
                    + "DESCRIPTION VARCHAR(350), USERNAME VARCHAR(20), "
                    + "ICON VARCHAR(50), REGISTERED DATETIME, "
                    + "PROFILEVIEW NUMERIC(8,0), TOPIC_NAME VARCHAR(50), "
                    + "SHORT_TOPIC VARCHAR(30), CONTAINER_NAME VARCHAR(50), "
                    + "SHORT_CONTAINER VARCHAR(30));");
            HostApp.refresh();
        }

        try
        {
            // count the number of stories
            m_http_request.setAsync(false);
            m_total_count = getTotalStoriesCount();
            m_job.setMaximum(m_total_count);

            // start at the end
            m_request_end_date = m_overall_end_date;
            m_request_offset = 0;
            m_lastids = [];

            // start the timer
            m_http_request.setAsync(true);
            m_request_timer.start();
            m_request_timer.tick.fire();
        }
        catch(e)
        {
            // cancel the job due to an error
            cancelJob();
        }

    }


    // returns the total number of stories that will occur (according
    // to the current parameters
    function getTotalStoriesCount()
    {
        // since the server limits the max date range to 30 days, break
        // ranges bigger than that into chunks
        var chunk_start = m_overall_start_date,
            total = 0,
            offset = 0;

        // get all of the "full" chunks, starting from the beginning
        while (chunk_start + m_chunk_size < m_overall_end_date)
        {
            var chunk_end = chunk_start + m_chunk_size,
                params = {count: 0,
                          offset: offset,
                          min_submit_date: chunk_start,
                          max_submit_date: chunk_end};
            total += +_parseResponse(m_http_request.get(
                                     _makeBaseUrl(params))).total;
            chunk_start = chunk_end;
            offset = 1;
            
            if (m_job.isCancelling())
                throw new Error;
        }

        // get the last (partial) chunk
        var params = {count: 0,
                      offset: 1,
                      min_submit_date: chunk_start,
                      max_submit_date: m_overall_end_date};
        total += +_parseResponse(m_http_request.get(
                                 _makeBaseUrl(params))).total;

        return total;
    }


    // --- "private" member functions ---


    // requests the next chunk of stories
    function _addStoryRequest()
    {
        // clear out the event queue
        if (!m_request_timer.isRunning())
            return;

        // throw out the old request if it timed out
        if (!m_http_request.isDone())
        {
            m_http_request.finished.disconnectAll();
            _initHttpRequest();
            m_http_request.setAsync(true);
            m_http_request.finished.connect(this, _handleStoryResponse);
        }

        // calculate the ceiling for the request
        var start_date = Math.max(m_request_end_date - m_chunk_size,
                                  m_overall_start_date);

        // check if done
        m_repeated++;
        if (start_date > m_request_end_date || m_repeated >= 3)
        {
            finishJob();
            return;
        }

        // send a new async request
        var params = {count: 100,
                      offset: m_request_offset,
                      min_submit_date: start_date,
                      max_submit_date: m_request_end_date};
        m_http_request.get(_makeBaseUrl(params));
    }


    // triggered when a response is received from the servers,
    // inserts the stories into the database
    function _handleStoryResponse()
    {
        try
        {
            // extract the response
            var obj = _parseResponse(m_http_request.getResponseText());
            var numStories = +obj.count;

            if (!numStories)
                return;

            // calculate the remaining time
            var min = (m_total_count - m_job.getValue()) / 600,
                minstr = "";
            for (var i = m_overall_start_date + 2592000; i < m_request_end_date; i += 2592000)
                min += 1/6; // add in month chunk breaks
            if (min <= 0.5)
                minstr = "<30 seconds";
            else if(min <= 1)
                minstr = "<1 minute";
            else if(min <= 60)
                minstr = Math.round(min) + " minutes";
            else
                minstr = (Math.round(min / 6) / 10) + " hours";

            // update the job status
            m_job.incrementValue(numStories);
            m_job.setTitle("Digg Import: " + comma(m_job.getValue())
                                            + "/" + comma(m_total_count)
                                            + " (est. " + minstr + ")");

            // insert the stories into the database
            var inserter = m_db.bulkInsert(m_output_tablename),
                newlastIds = [];
            if (!inserter)
            {
                cancelJob();
                throw new Error;
            }
            for (var i = 0; i < numStories; i++)
            {
                var item = obj.stories[i];
                newlastIds[item.id] = true;

                // make sure the server didn't give duplicate results
                if (m_lastids[item.id])
                    continue;

                inserter[0]  = item.id;
                inserter[1]  = new Date(item.submit_date*1000);
                inserter[2]  = item.status;
                inserter[3]  = item.diggs;
                inserter[4]  = item.comments;
                inserter[5]  = item.href;
                inserter[6]  = item.link;
                inserter[7]  = item.title;
                inserter[8]  = item.description;
                inserter[9]  = item.user.name;
                inserter[10] = item.user.icon;
                inserter[11] = new Date(item.user.registered*1000);
                inserter[12] = item.user.profileviews;
                inserter[13] = item.topic.name;
                inserter[14] = item.topic.short_name;
                inserter[15] = item.container.name;
                inserter[16] = item.container.short_name;
                inserter.insertRow();

                // move the next request's end date to the last story's
                // submit date
                if (m_request_end_date != item.submit_date)
                {
                    m_request_end_date = item.submit_date;
                    m_request_offset = 1;
                }
                // prevent overlap
                else
                    m_request_offset++;
            }
            inserter.finishInsert();
            m_lastids = newlastIds;
            m_repeated = 0;
        }
        // ignore this response so that it will be submitted again
        catch(e)
        {

        }

        // check if done
        if (m_job.getValue() + 1 >= m_job.getMaximum() || m_repeated >= 3)
            finishJob();
    }


    // updates the request url to include the category, status and username params
    function _makeRequestType()
    {
        if (m_username.length)
        {
            m_request_type = "user/" + encodeURIComponent(m_username) + "/"
                + (m_status && m_status != "all" ? m_status : "submissions");
        }
        else
        {
            var str = "stories";

            if (m_category && m_category != "all")
                str += "/container/" + m_category;

            if (m_status && m_status != "all")
                str += "/" + m_status;

            m_request_type = str;
        }
    }

}






function comma(n)
{
    var s = new Number(n).toString();
    var p = s.indexOf('.');
    p = (p == -1 ? s.length : p) - 3;

    while(p > 0)
    {
        s = s.substr(0, p) + ',' + s.substr(p, s.length - p);
        p -= 3;
    }
    return s;
}

