/*---------------------------------------------------------------------------

    Filename: digg_import_ui.js
    Project:  Digg Import
    (C) Copyright 2007, Kirix Corporation, All Rights Reserved.

    Summary:         Digg Import User Interface
    Developer(s):    David Z. Williams
                     Jeff Engel

---------------------------------------------------------------------------*/


class DiggImportForm extends Form
{
    // -- member variables --
    var m_db;
    var m_importer;
    var m_category_choicebox;
    var m_status_choicebox;
    var m_username_textbox;
    var m_start_datepicker;
    var m_end_datepicker;
    var m_output_textbox;
    var m_append_checkbox;

    // -- functions --

    function DiggImportForm()
    {
        super("Digg Import");
        setClientSize(350, 237);
        setMinSize(getSize());
        setMaxSize(500, getSize().height);
        center();
        m_db = new DbConnection;
        
        var now = new Date();


        // category layout
        m_category_choicebox = new ChoiceBox();
        m_category_choicebox.add("All");
        m_category_choicebox.select(0);
        var category_layout = new BoxLayout(Layout.Horizontal);
        category_layout.add(new Label("Category:", 0, 0, 60), 0, Layout.Center);
        category_layout.add(m_category_choicebox, 1);

        // status layout
        m_status_choicebox = new ChoiceBox();
        m_status_choicebox.selectionChanged.connect(this, onStatusChanged);
        m_status_choicebox.add("All");
        m_status_choicebox.add("Popular");
        m_status_choicebox.add("Upcoming");
        m_status_choicebox.select(0);
        var status_layout = new BoxLayout(Layout.Horizontal);
        status_layout.add(new Label("Status:", 0, 0, 60), 0, Layout.Center);
        status_layout.add(m_status_choicebox, 1);

        // username layout
        m_username_textbox = new TextBox();
        m_username_textbox.textChanged.connect(this, onUsernameChanged);
        var username_layout = new BoxLayout(Layout.Horizontal);
        username_layout.add(new Label("Username:", 0, 0, 60), 0, Layout.Center);
        username_layout.add(m_username_textbox, 1, Layout.Expand);

        // start date layout
        m_start_datepicker = new DatePicker(now, 2004, now.getFullYear() + 1);
        var startdate_layout = new BoxLayout(Layout.Horizontal);
        startdate_layout.add(new Label("Start Date:", 0, 0, 60), 0, Layout.Center);
        startdate_layout.add(m_start_datepicker, 1);

        // end date layout
        m_end_datepicker = new DatePicker(now, 2004, now.getFullYear() + 1);
        var enddate_layout = new BoxLayout(Layout.Horizontal);
        enddate_layout.add(new Label("End Date:", 0, 0, 60), 0, Layout.Center);
        enddate_layout.add(m_end_datepicker, 1, Layout.Expand);

        // output layout
        m_output_textbox = new TextBox("/digg/stories");
        m_append_checkbox = new CheckBox("Append");
        var output_layout = new BoxLayout(Layout.Horizontal);
        output_layout.add(new Label("Output:", 0, 0, 60), 0, Layout.Center);
        output_layout.add(m_output_textbox, 1, Layout.Expand);
        output_layout.addSpacer(8);
        output_layout.add(m_append_checkbox, 0, Layout.Center);

        // button layout
        var start_button = new Button("Start");
        start_button.setDefault();
        start_button.click.connect(this, onStartClicked);
        var close_button = new Button("Close");
        close_button.click.connect(this, onCloseClicked);

        var button_layout = new BoxLayout(Layout.Horizontal);
        button_layout.addStretchSpacer();
        button_layout.add(start_button);
        button_layout.addSpacer(8);
        button_layout.add(close_button);


        // create main layout
        var main_layout = new BoxLayout(Layout.Vertical);
        main_layout.addSpacer(10);
        main_layout.add(category_layout, 0, Layout.Expand | Layout.Left | Layout.Right, 10);
        main_layout.addSpacer(10);
        main_layout.add(status_layout, 0, Layout.Expand | Layout.Left | Layout.Right, 10);
        main_layout.addSpacer(10);
        main_layout.add(username_layout, 0, Layout.Expand | Layout.Left | Layout.Right, 10);
        main_layout.addSpacer(10);
        main_layout.add(startdate_layout, 0, Layout.Expand | Layout.Left | Layout.Right, 10);
        main_layout.addSpacer(10);
        main_layout.add(enddate_layout, 0, Layout.Expand | Layout.Left | Layout.Right, 10);
        main_layout.addSpacer(10);
        main_layout.add(output_layout, 0, Layout.Expand | Layout.Left | Layout.Right, 10);
        main_layout.addStretchSpacer();
        main_layout.add(new Line(), 0, Layout.Expand);
        main_layout.addSpacer(8);
        main_layout.add(button_layout, 0, Layout.Expand | Layout.Left | Layout.Right, 8);
        main_layout.addSpacer(8);
        setLayout(main_layout);

        show();
        updateCategories();
    }
    
    function onStatusChanged()
    {
        var idx = m_status_choicebox.getSelection();
        m_start_datepicker.setEnabled(idx != 3 && idx != 4);
        m_end_datepicker.setEnabled(idx != 3 && idx != 4);
    }
    
    function onUsernameChanged()
    {
        m_category_choicebox.setEnabled(!m_username_textbox.getText().length);
    }

    function onStartClicked()
    {
        
        var now = new Date(),
            startdate = m_start_datepicker.getValue(),
            enddate = m_end_datepicker.getValue();
        enddate.setDate(enddate.getDate() + 1); // move to midnight

        if (startdate.getTime() >= now.getTime())
        {
            alert("Error: The start date cannot be in the future");
            return;
        }
        else if (enddate.getTime() <= startdate.getTime())
        {
            alert("Error: The end date must be after the start date");
            return;
        }
        else if (startdate.getFullYear() == 2004 && startdate.getMonth() != 11)
        {
            alert("Error: No stories.  Digg wasn't founded yet!");
            return;
        }
        else if (!m_append_checkbox.getValue() && m_db.exists(
                m_output_textbox.getText()) && !confirm("You are about to "
                + "overwrite the table '" + m_output_textbox.getText()
                + "' which will erase all existing data.\n\nAre you sure you "
                + "want to continue?"))
        {
            m_output_textbox.selectAll();
            return;
        }

        show(false);

        // run the stories request
        m_importer = new DiggStoryImport;
        m_importer.finished.connect(this, onCloseClicked);
        m_importer.setCategory(m_category_choicebox.getString(m_category_choicebox.getSelection()));
        m_importer.setStatus(m_status_choicebox.getString(m_status_choicebox.getSelection()));
        m_importer.setUsername(m_username_textbox.getText());
        m_importer.setStartDate(startdate);
        m_importer.setEndDate(enddate);
        m_importer.setAppendData(m_append_checkbox.getValue());
        m_importer.setOutputTable(m_output_textbox.getText());
        m_importer.run();
    }

    function onCloseClicked()
    {
        close();
    }

    function updateCategories()
    {
        try
        {
            var importer = new DiggImportBase("topics"),
                topics = importer._parseResponse(importer.m_http_request.get(
                            importer._makeBaseUrl())).topics,
                containers = [];

            // sort the topics into unique containers
            for(var i in topics)
                containers[topics[i].container.short_name] = topics[i].container.name;

            // add the containers as categories
            for (var i in containers)
                m_category_choicebox.add(containers[i]);
        }
        catch(e)
        {
            alert("Could not connect to the Digg servers to fetch the "
                + "categories.  Please check your internet connection before "
                + "continuing.");
        }
    }
}
