/*---------------------------------------------------------------------------

    Filename: confirm.js
    Project:  Controls
    (C) Copyright 2008, Kirix Corporation, All Rights Reserved.

    Summary:         Displays a Confirm dialog box with the specified message
                     and OK and Cancel buttons.
    Developer(s):    Jeff Engel

---------------------------------------------------------------------------*/



// -- example 1: text used in the dialog --

/*

// text to display in the Confirm dialog
var text = "Are you sure you want to continue?";

// display the dialog and store the result
var result = confirm(text);

if (result == true)
    alert("You pressed yes!");
else
    alert("You pressed no!");

// start the main application loop
Application.run();

*/



// -- example 2: form used in the dialog --

/*

// checkbox to use within the form
var checkbox = new CheckBox("Don't give an alert", 0, 30);

// form to display in the Confirm dialog
var form = new Form("", 0, 0, -1, 80);
form.add(new Label("Are you sure you want to continue?"));
form.add(checkbox);

// display the dialog and store the result
var result = confirm(form);

if (!checkbox.getValue())
{
    if (result == true)
        alert("You pressed yes!");
    else
        alert("You pressed no!");
}

// start the main application loop
Application.run();

*/





// (METHOD) confirm
// Description: Displays a Confirm dialog box with the specified message and OK
//              and Cancel buttons.
//
// Syntax: confirm(message : String) : Boolean
// Syntax: confirm(message : String,
//                 title : String) : Boolean
//
// Remarks: Displays a Confirm dialog box with the specified message and OK and
//          Cancel buttons.  The function will return true iff the Yes button is
//          pressed, and false otherwise.
//
// Param(text): The |text| to display in the dialog.
// Param(title): The |title| of the dialog.
//
// Returns: |true| if the Yes button was pressed, and |false| if the No button
//          was pressed or the dialog was closed prematurely.
function confirm(contents, title)
{
    if (title === undefined)
        title = "Kirix Strata";

    // --- top layout ---

    var pic = new PictureBox(0, 0, 32, 32);
    pic.setImage(HostBitmap.get("exclamation", 32));

    var obj;
    if (typeof(contents) == "string")
    {
        obj = new Label(contents, 0, 0, 277);
        obj.wrap(245);
    }
    else
    {
        obj = contents;
        obj.setSize(277, obj.getSize().height);
    }

    var top_layout = new BoxLayout(Layout.Horizontal);
    top_layout.addSpacer(30);
    top_layout.add(pic, 0, Layout.Top, 30);
    top_layout.addSpacer(8);
    top_layout.add(obj, 0, Layout.Top, 30);
    top_layout.addSpacer(30);

    var top_form = new Form("", 0, 0, 345);
    top_form.setBackgroundColor(new Color(255, 255, 255));
    top_form.setLayout(top_layout);

    // --- bottom layout ---

    var yes_btn = new Button("Yes");
    var no_btn = new Button("No");
    
    var button_layout = new BoxLayout(Layout.Horizontal);
    button_layout.addStretchSpacer();
    button_layout.add(yes_btn, 0, Layout.Top | Layout.Bottom | Layout.Right, 8);
    button_layout.add(no_btn, 0, Layout.Top | Layout.Bottom | Layout.Right, 8);

    // --- main layout ---

    var main_layout = new BoxLayout(Layout.Vertical);
    main_layout.add(top_form, 1, Layout.Expand);
    main_layout.add(button_layout, 0, Layout.Expand);

    var height = obj.getSize().height + 60;
    if (height < 92)
        height = 92;
    else if (height > 1000)
        height = 720;
    height += yes_btn.getSize().height + 16;

    var main_form = new Form(title);
    main_form.setClientSize(345, height);
    main_form.setMinSize(main_form.getSize());
    main_form.setMaxSize(main_form.getSize());
    main_form.setLayout(main_layout);
    main_form.center();

    // --- show dialog ---

    confirm.form = main_form;
    yes_btn.click.connect(function() {
                confirm.form.endDialog(DialogResult.OK); });
    no_btn.click.connect(function() {
                confirm.form.endDialog(DialogResult.Cancel); });
    System.beep();
    var ret = main_form.showDialog() == DialogResult.OK;
    confirm.form = undefined;

    return ret;
}
