
/*---------------------------------------------------------------------------

    Filename: datepicker.js
    Project:  Controls
    (C) Copyright 2008, Kirix Corporation, All Rights Reserved.

    Summary:        Control for picking dates in a form

    Developer(s):   David Z. Williams

---------------------------------------------------------------------------*/


class DatePicker extends BoxLayout
{
    // events
    var dateChanged;
    
    // controls
    var m_year_choice;
    var m_month_choice;
    var m_day_choice;
    
    function DatePicker(date, low_year, high_year)
    {
        super(Layout.Horizontal);
        
        // initialize the 'date_changed' event
        dateChanged = new Event;
        
        // if a default date wasn't passed, use the current date
        if (!date)
            date = new Date;
        
        m_year_choice = new ChoiceBox(0,0, 60);
        m_year_choice.selectionChanged.connect(this, onYearChanged);
        
        m_month_choice = new ChoiceBox(0,0, 85);
        m_month_choice.selectionChanged.connect(this, onMonthChanged);
        
        m_day_choice = new ChoiceBox(0,0, 45);
        m_day_choice.selectionChanged.connect(this, onDayChanged);
        
        populateYearChoice(date.getFullYear(), low_year, high_year);
        populateMonthChoice(date.getMonth() + 1);
        populateDayChoice(date.getDate());
        
        add(m_month_choice, 4, Layout.Expand);
        addSpacer(4);
        add(m_day_choice, 2, Layout.Expand);
        addSpacer(4);
        add(m_year_choice, 3, Layout.Expand);
    }
    
    function getValue()
    {
        return new Date(+m_year_choice.getString(m_year_choice.getSelection()),
                        m_month_choice.getSelection(),
                        m_day_choice.getSelection()+1);
    }
    
    // -- "private" functions --
    
    function populateMonthChoice(default_val)
    {
        m_month_choice.add("January");
        m_month_choice.add("February");
        m_month_choice.add("March");
        m_month_choice.add("April");
        m_month_choice.add("May");
        m_month_choice.add("June");
        m_month_choice.add("July");
        m_month_choice.add("August");
        m_month_choice.add("September");
        m_month_choice.add("October");
        m_month_choice.add("November");
        m_month_choice.add("December");

        m_month_choice.select(default_val - 1);
    }
    
    function populateDayChoice(default_val)
    {
        // freeze
        m_day_choice.disablePaint();

        // calculate the last day of the month
        var d = new Date(m_year_choice.getSelection() + 1900, m_month_choice.getSelection() + 1, 0);
        var oldcount = m_day_choice.getCount();
        var newcount = d.getDate();

        // add values we don't already have
        if(newcount > oldcount)
            for (var day = m_day_choice.getCount() + 1; day <= newcount; day++)
                m_day_choice.add(day);
        // delete extra values
        else if (newcount < oldcount)
            for (var day = oldcount; day > newcount; day--)
                m_day_choice.delete(day - 1);

        // select the default value
        if (default_val >= newcount)
            default_val = newcount;
        m_day_choice.select(default_val - 1);

        // thaw
        m_day_choice.enablePaint();
    }
    
    function populateYearChoice(default_val, low_val, high_val)
    {
        if (default_val < low_val)
            low_val = default_val;
        if (default_val > high_val)
            high_val = default_val;

        for (var i = low_val; i <= high_val; i++)
            m_year_choice.add(i);

        m_year_choice.select(m_year_choice.findString(default_val));
    }
    
    function onYearChanged()
    {
        populateDayChoice(m_day_choice.getSelection() + 1);
        dateChanged.fire(this, getValue());
    }
    
    function onMonthChanged()
    {
        populateDayChoice(m_day_choice.getSelection() + 1);
        dateChanged.fire(this, getValue());
    }
    
    function onDayChanged()
    {
        dateChanged.fire(this, getValue());
    }

    function setEnabled(flag)
    {
        m_month_choice.setEnabled(flag);
        m_day_choice.setEnabled(flag);
        m_year_choice.setEnabled(flag);
    }
}
